@@ -11,6 +11,12 @@
 (load "src/cold/defun-load-or-cload-xcompiler.lisp")
 (load-or-cload-xcompiler #'host-load-stem)
 
+(let ((*features* (cons :sb-xc *features*)))
+  (load "src/cold/muffler.lisp"))
+(setq sb!c::*handled-conditions*
+      '(((or (satisfies unable-to-optimize-note-p)
+             sb!ext:code-deletion-note) . muffle-warning)))
+
 (defun proclaim-target-optimization ()
   (let ((debug (if (position :sb-show *shebang-features*) 2 1)))
     (sb-xc:proclaim
@@ -71,14 +77,6 @@
 (setf *in-target-compilation-mode-fn* #'in-target-cross-compilation-mode)
 
 ;;; Run the cross-compiler to produce cold fasl files.
-;; This suppresses ~6000 lines of "undefined function" warnings from the
-;; cross-compiler stemming from the calls to INSTANCE-TYPEP that occur before
-;; src/code/class gets compiled. It magically converts efficiently,
-;; but IR1 is a little bit naive about how it happens.
-(dolist (f '(sb!kernel:layout-depthoid
-             sb!kernel:layout-inherits))
-  (setf (sb!int:info :function :kind f) :function
-        (sb!int:info :function :where-from f) :declared))
 ;; ... and since the cross-compiler hasn't seen a DEFMACRO for QUASIQUOTE,
 ;; make it think it has, otherwise it fails more-or-less immediately.
 (setf (sb!int:info :function :kind 'sb!int:quasiquote) :macro
@@ -127,7 +125,7 @@
   (loop for (type-spec . (count . interr-p)) in l
         do (format t "~:[ ~;+~] ~5D ~S~%" interr-p count type-spec))
   (format t "~&Error numbers not used by checkgen:~%")
-  (loop for (spec . symbol) across sb!c::*backend-internal-errors*
+  (loop for (spec . symbol) across sb!c:+backend-internal-errors+
         when (and (not (stringp spec))
                   (not (gethash spec sb!c::*checkgen-used-types*)))
         do (format t "       ~S~%" spec)))
