@@ -27,7 +27,9 @@
     ;; is free to :USE (PACKAGE-USE-LIST :CL-USER) anyway.:-|
     nil))
 
-(defmacro defpackage (package &rest options)
+;; "mundanely" because this macro can't work (never has, never will)
+;; until the target system is fully operational.
+(defmacro-mundanely defpackage (package &rest options)
   #!+sb-doc
   #.(format nil
   "Defines a new package called PACKAGE. Each of OPTIONS should be one of the
@@ -153,8 +155,9 @@
        (%defpackage ,(stringify-string-designator package) ',nicknames ',size
                     ',shadows ',shadowing-imports ',(if use-p use :default)
                     ',imports ',interns ',exports ',implement ',local-nicknames
-                    ',lock ',doc
-                    (sb!c:source-location)))))
+                    ',lock (sb!c:source-location)
+                    ,@(and doc
+                           `(,doc))))))
 
 (defun check-disjoint (&rest args)
   ;; An arg is (:key . set)
@@ -264,6 +267,7 @@
 
 (declaim (type list *on-package-variance*))
 (defvar *on-package-variance* '(:warn t)
+  #!+sb-doc
   "Specifies behavior when redefining a package using DEFPACKAGE and the
 definition is in variance with the current state of the package.
 
@@ -382,13 +386,12 @@ specifies to signal a warning if SWANK package is in variance, and an error othe
 
 (defun %defpackage (name nicknames size shadows shadowing-imports
                     use imports interns exports implement local-nicknames
-                    lock doc-string
-                    source-location)
+                    lock source-location &optional doc)
   (declare (type simple-string name)
            (type list nicknames shadows shadowing-imports
                  imports interns exports)
            (type (or list (member :default)) use)
-           (type (or simple-string null) doc-string))
+           (type (or simple-string null) doc))
   (with-package-graph ()
     (let* ((existing-package (find-package name))
            (use (use-list-packages existing-package use))
@@ -400,7 +403,7 @@ specifies to signal a warning if SWANK package is in variance, and an error othe
                                         shadows shadowing-imports
                                         use imports interns exports
                                         implement local-nicknames
-                                        lock doc-string)
+                                        lock doc)
           (let ((package (make-package name
                                        :use nil
                                        :internal-symbols (or size 10)
@@ -411,7 +414,7 @@ specifies to signal a warning if SWANK package is in variance, and an error othe
                             shadows shadowing-imports
                             use imports interns exports
                             implement local-nicknames
-                            lock doc-string))))))
+                            lock doc))))))
 
 (defun find-or-make-symbol (name package)
   (multiple-value-bind (symbol how) (find-symbol name package)
