@@ -141,7 +141,7 @@
 (sb!xc:deftype logical-host-designator ()
   '(or host string))
 
-(sb!xc:deftype package-designator () '(or string-designator sb!xc:package))
+(sb!xc:deftype package-designator () '(or string-designator package))
 ;;; a designator for a list of symbols
 (sb!xc:deftype symbols-designator () '(or list symbol))
 
@@ -155,7 +155,7 @@
 
 ;;; a consed sequence result. If a vector, is a simple array.
 (sb!xc:deftype consed-sequence ()
-  '(or (simple-array * (*)) (and sequence (not vector))))
+  '(or (simple-array * (*)) list extended-sequence))
 
 ;;; the :END arg to a sequence
 (sb!xc:deftype sequence-end () '(or null index))
@@ -167,9 +167,10 @@
 ;;; a valid argument to a stream function
 (sb!xc:deftype stream-designator () '(or stream (member nil t)))
 
-;;; an object suitable for input to standard functions that accept
-;;; "environment objects" (of the ANSI glossary)
-(sb!xc:deftype lexenv-designator () '(or lexenv null))
+;;; something valid as the :EXTERNAL-FORMAT argument to OPEN, LOAD,
+;;; COMPILE-FILE and friends.
+(sb!xc:deftype external-format-designator ()
+  '(or keyword (cons keyword)))
 
 ;;; a thing that can be passed to FUNCALL & friends
 ;;;
