@@ -260,10 +260,8 @@
                (list (subseq buffer 0 size)
                      (undefined-package-error package)))
         (push-fop-table (without-package-locks
-                          (intern* buffer
-                                   size
-                                   package
-                                   :no-copy t)) fasl-input))))
+                          (%intern buffer size package nil))
+                        fasl-input))))
 
 (!define-fop 80 (fop-lisp-symbol-save ((:operands namelen)))
   (aux-fop-intern namelen *cl-package* (fasl-input)))
@@ -280,17 +278,29 @@
 (!define-fop #xF0 (fop-symbol-in-package-save ((:operands pkg-index namelen)))
   (aux-fop-intern namelen (ref-fop-table (fasl-input) pkg-index) (fasl-input)))
 
-(!define-fop 96 (fop-uninterned-symbol-save ((:operands namelen)))
-  (let ((res (make-string namelen)))
-    #!-sb-unicode
-    (read-string-as-bytes (fasl-input-stream) res)
-    #!+sb-unicode
-    (read-string-as-unsigned-byte-32 (fasl-input-stream) res)
-    (push-fop-table (make-symbol res) (fasl-input))))
+;;; Symbol-hash is usually computed lazily and memoized into a symbol.
+;;; Laziness slightly improves the speed of allocation.
+;;; But when loading fasls, the time spent in the loader totally swamps
+;;; any time savings of not precomputing symbol-hash.
+;;; INTERN hashes everything anyway, so let's be consistent
+;;; and precompute the hashes of uninterned symbols too.
+(macrolet ((ensure-hashed (symbol-form)
+             `(let ((symbol ,symbol-form))
+                (ensure-symbol-hash symbol)
+                symbol)))
+  (!define-fop 96 (fop-uninterned-symbol-save ((:operands namelen)))
+    (let ((res (make-string namelen)))
+      #!-sb-unicode
+      (read-string-as-bytes (fasl-input-stream) res)
+      #!+sb-unicode
+      (read-string-as-unsigned-byte-32 (fasl-input-stream) res)
+      (push-fop-table (ensure-hashed (make-symbol res))
+                      (fasl-input))))
 
-(!define-fop 104 (fop-copy-symbol-save ((:operands table-index)))
-  (push-fop-table (copy-symbol (ref-fop-table (fasl-input) table-index))
-                  (fasl-input)))
+  (!define-fop 104 (fop-copy-symbol-save ((:operands table-index)))
+    (push-fop-table (ensure-hashed
+                     (copy-symbol (ref-fop-table (fasl-input) table-index)))
+                    (fasl-input))))
 
 (!define-fop 44 (fop-package (pkg-designator))
   (find-undeleted-package-or-lose pkg-designator))
@@ -567,7 +577,7 @@
 
 ;; this gets you an #<fdefn> object, not the result of (FDEFINITION x)
 (!define-fop 60 (fop-fdefn (name))
-  (awhen (deprecated-thing-p :function name) ; returns the stage of deprecation
+  (awhen (deprecated-thing-p 'function name) ; returns the stage of deprecation
     (pushnew (list* it name :function)
              (%fasl-input-deprecated-stuff (fasl-input)) :test 'equal))
   (find-or-create-fdefn name))
