@@ -14,28 +14,27 @@
 
 ;;;; internal errors
 
-(defvar *internal-errors*
-  #.(map 'vector
-         (lambda (x) (if (typep (car x) '(or symbol cons)) (car x) 0))
-         sb!c:*backend-internal-errors*))
+(macrolet ((def-it ()
+             (let ((n (1+ (position-if 'stringp sb!c:+backend-internal-errors+
+                                       :key #'car :from-end t))))
+               `(progn
+                  (declaim ((simple-vector ,n) **internal-error-handlers**))
+                  (defglobal **internal-error-handlers**
+                    (make-array ,n :initial-element 0))))))
+  (def-it))
 
 (eval-when (:compile-toplevel :execute)
 (sb!xc:defmacro deferr (name args &rest body)
-  (let ((n (length args)))
-    (unless (<= n 3)
-      (error "Update (DEFUN INTERNAL-ERROR) for ~D error arguments" n)))
-  `(progn
-     (setf (svref *internal-errors* ,(error-number-or-lose name))
-         (lambda (name ,@args)
+  (multiple-value-bind (llks required optional rest) (parse-lambda-list args)
+    (declare (ignore llks))
+    (aver (not rest))
+    (let ((max (+ (length required) (length optional))))
+      (unless (<= max 3)
+        (error "Update (DEFUN INTERNAL-ERROR) for ~D error arguments" max))))
+  `(setf (svref **internal-error-handlers** ,(error-number-or-lose name))
+         (named-lambda ,(string name) (name ,@args)
            (declare (optimize (sb!c::verify-arg-count 0)) (ignorable name))
-           ,@body))
-     ;; general KLUDGE: refer to each error symbol. Can't just return ',NAME
-     ;; - the compiler knows an effectless form when it sees one.
-     (locally (declare (notinline string)) (string ',name))))
-) ; EVAL-WHEN
-
-;; special KLUDGE for UNKNOWN-ERROR, which has no DEFERR at all
-(locally (declare (notinline string)) (string 'unknown-error))
+           ,@body)))) ; EVAL-WHEN
 
 (deferr undefined-fun-error (fdefn-or-symbol)
   (error 'undefined-function
@@ -211,8 +210,9 @@
 ;;; Returns true if number of arguments matches required/optional
 ;;; arguments handler expects.
 (defun internal-error-args-ok (arguments handler)
-  (multiple-value-bind (req opt)
+  (multiple-value-bind (llks req opt)
       (parse-lambda-list (%simple-fun-arglist handler) :silent t)
+    (declare (ignore llks))
     ;; The handler always gets name as the first (extra) argument.
     (let ((n (1+ (length arguments)))
           (n-req (length req))
@@ -244,19 +244,19 @@
           (%primitive sb!c:current-binding-pointer)))
    (infinite-error-protect
     (/show0 "about to bind ALIEN-CONTEXT")
-    (let* ((alien-context (locally
-                              (declare (optimize (inhibit-warnings 3)))
-                            (sap-alien context (* os-context-t))))
+    (let* ((alien-context (sap-alien context (* os-context-t)))
            #!+c-stack-is-control-stack
-           (*saved-fp-and-pcs*
-             (cons (cons (%make-lisp-obj (sb!vm:context-register
-                                          alien-context
-                                          sb!vm::cfp-offset))
-                         (sb!vm:context-pc alien-context))
-                   (when (boundp '*saved-fp-and-pcs*)
-                     *saved-fp-and-pcs*))))
-      (declare (truly-dynamic-extent *saved-fp-and-pcs*))
-      (/show0 "about to bind ERROR-NUMBER and ARGUMENTS")
+           (fp-and-pc (make-array 2 :element-type 'word)))
+      #!+c-stack-is-control-stack
+      (declare (truly-dynamic-extent fp-and-pc))
+      #!+c-stack-is-control-stack
+      (setf (aref fp-and-pc 0) (sb!vm:context-register alien-context sb!vm::cfp-offset)
+            (aref fp-and-pc 1) (sb!sys:sap-int (sb!vm:context-pc alien-context)))
+      (let (#!+c-stack-is-control-stack
+            (*saved-fp-and-pcs* (cons fp-and-pc *saved-fp-and-pcs*)))
+        #!+c-stack-is-control-stack
+        (declare (truly-dynamic-extent *saved-fp-and-pcs*))
+       (/show0 "about to bind ERROR-NUMBER and ARGUMENTS"))
       (multiple-value-bind (error-number arguments)
           (sb!vm:internal-error-args alien-context)
         (with-interrupt-bindings
@@ -265,10 +265,21 @@
             (/show0 "back from FIND-INTERRUPTED-NAME")
             (let ((*current-internal-error* error-number)
                   (fp (int-sap (sb!vm:context-register alien-context
-                                                       sb!vm::cfp-offset)))
-                  (handler (and (< -1 error-number (length *internal-errors*))
-                                (svref *internal-errors* error-number))))
-              (cond ((and (functionp handler)
+                                                       sb!vm::cfp-offset))))
+              (if (and (>= error-number (length **internal-error-handlers**))
+                       (< error-number (length sb!c:+backend-internal-errors+)))
+                  (error 'type-error
+                         :datum (sb!di::sub-access-debug-var-slot
+                                 fp (first arguments) alien-context)
+                         :expected-type
+                         (car (svref sb!c:+backend-internal-errors+
+                                     error-number)))
+                  (let ((handler
+                         (and (typep error-number
+                                     '#.`(mod ,(length **internal-error-handlers**)))
+                              (svref **internal-error-handlers** error-number))))
+                   (cond
+                    ((and (functionp handler)
                           (internal-error-args-ok arguments handler))
                      (macrolet ((arg (n)
                                   `(sb!di::sub-access-debug-var-slot
@@ -278,11 +289,6 @@
                          (1 (funcall handler name (arg 0)))
                          (2 (funcall handler name (arg 0) (arg 1)))
                          (3 (funcall handler name (arg 0) (arg 1) (arg 2))))))
-                    ((typep handler '(or symbol cons))
-                     (error 'type-error
-                            :datum (sb!di::sub-access-debug-var-slot
-                                    fp (first arguments) alien-context)
-                            :expected-type handler))
                     ((eql handler 0) ; if (DEFERR x) was inadvertently omitted
                      (error 'simple-error
                             :format-control
@@ -302,7 +308,7 @@
                                   (mapcar (lambda (sc-offset)
                                             (sb!di::sub-access-debug-var-slot
                                              fp sc-offset alien-context))
-                                          arguments)))))))))))))
+                                          arguments)))))))))))))))
 
 (defun control-stack-exhausted-error ()
   (let ((sb!debug:*stack-top-hint* nil))
