@@ -390,23 +390,18 @@
 
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
   ;;; Assign SETF macro information for NAME, making all appropriate checks.
-  (macrolet ((assign-it ()
+ (macrolet ((assign-it ()
                `(progn
                   (when inverse
                     (clear-info :setf :expander name)
                     (setf (info :setf :inverse name) inverse))
                   (when expander
-                    #-sb-xc-host
-                    (setf (%fun-lambda-list
-                           (if (listp expander) (cdr expander) expander))
-                          expander-lambda-list)
                     (clear-info :setf :inverse name)
                     (setf (info :setf :expander name) expander))
                   (when doc
                     (setf (fdocumentation name 'setf) doc))
                   name)))
-  (defun %defsetf (name expander expander-lambda-list inverse doc)
-    #+sb-xc-host (declare (ignore expander-lambda-list))
+  (defun %defsetf (name expander inverse &optional doc)
     (with-single-package-locked-error
         (:symbol name "defining a setf-expander for ~A"))
     (let ((setf-fn-name `(setf ,name)))
@@ -433,8 +428,8 @@
            (style-warn "defining setf macro for ~S when ~S is also defined"
                        name setf-fn-name)))))
     (assign-it))
-  (defun !quietly-defsetf (name expander expander-lambda-list inverse doc)
-    #+sb-xc-host (declare (ignore expander-lambda-list))
+  ;; For cold-init, because any warning will cause a crash.
+  (defun !quietly-defsetf (name expander inverse &optional doc)
     (assign-it))))
 
 (def!macro sb!xc:defsetf (access-fn &rest rest)
@@ -447,7 +442,7 @@
   (typecase rest
     ((cons (and symbol (not null)) (or null (cons string null)))
      `(eval-when (:load-toplevel :compile-toplevel :execute)
-        (%defsetf ',access-fn nil nil ',(car rest) ',(cadr rest))))
+        (%defsetf ',access-fn nil ',(car rest) ,@(cdr rest))))
     ((cons list (cons list))
      (destructuring-bind (lambda-list (&rest stores) &body body) rest
        (binding* (((llks req opt rest key aux env)
@@ -467,13 +462,15 @@
          `(eval-when (:compile-toplevel :load-toplevel :execute)
             (%defsetf ',access-fn
                       (cons ,(length stores)
-                            (lambda (,subforms ,env-var ,@stores)
+                            (named-lambda (%defsetf ,access-fn)
+                                          (,subforms ,env-var ,@stores)
+                              (declare (sb!c::lambda-list ,lambda-list))
                               ,@(if outer-decls (list outer-decls))
                               ,@(unless env `((declare (ignore ,env-var))))
                               (apply (lambda ,lambda-list
                                        ,@inner-decls (block ,access-fn ,@forms))
                                      ,subforms)))
-                      ',lambda-list nil ',doc)))))
+                      nil ,@(and doc `(,doc)))))))
     (t
      (error "Ill-formed DEFSETF for ~S" access-fn))))
 
@@ -577,15 +574,14 @@
   (unless (symbolp access-fn)
     (error "~S access-function name ~S is not a symbol."
            'sb!xc:define-setf-expander access-fn))
-  (with-unique-names (whole environment)
-    (multiple-value-bind (body local-decs doc)
-        (parse-defmacro lambda-list whole body access-fn
-                        'sb!xc:define-setf-expander
-                        :environment environment)
-      `(eval-when (:compile-toplevel :load-toplevel :execute)
-         (%defsetf ',access-fn
-                   (lambda (,whole ,environment) ,@local-decs ,body)
-                   ',lambda-list nil ',doc)))))
+  (multiple-value-bind (def doc)
+      ;; Perhaps it would be more elegant to keep the docstring attached
+      ;; to the expander function, as for CAS?
+      (make-macro-lambda `(setf-expander ,access-fn) lambda-list body
+                         'sb!xc:define-setf-expander access-fn
+                         :doc-string-allowed :external)
+    `(eval-when (:compile-toplevel :load-toplevel :execute)
+       (%defsetf ',access-fn ,def nil ,@(and doc `(,doc))))))
 
 (sb!xc:define-setf-expander values (&rest places &environment env)
   (declare (type sb!c::lexenv env))
@@ -736,7 +732,7 @@ place with bits from the low-order end of the new value."
   "The first argument is a byte specifier. The second is any place form
 acceptable to SETF. Replaces the specified byte of the number in this place
 with bits from the corresponding position in the new value."
-  (setf-expand-ldb bytespec place env 'deposit-byte 'mask-field))
+  (setf-expand-ldb bytespec place env 'deposit-field 'mask-field))
 
 (defun setf-expand-the (the type place env)
   (declare (type sb!c::lexenv env))
