@@ -111,6 +111,9 @@
                           :initial-element 0))
       (%make-package-hashtable table size))))
 
+(declaim (inline pkg-symbol-valid-p))
+(defun pkg-symbol-valid-p (x) (not (fixnump x)))
+
 ;;; Destructively resize TABLE to have room for at least SIZE entries
 ;;; and rehash its existing entries.
 (defun resize-package-hashtable (table size)
@@ -577,8 +580,7 @@ REMOVE-PACKAGE-LOCAL-NICKNAME, and the DEFPACKAGE option :LOCAL-NICKNAMES."
 ;;; Return a list of packages given a package designator or list of
 ;;; package designators, or die trying.
 (defun package-listify (thing)
-  (mapcar #'find-undeleted-package-or-lose
-          (if (listp thing) thing (list thing))))
+  (mapcar #'find-undeleted-package-or-lose (ensure-list thing)))
 
 ;;; ANSI specifies (in the definition of DELETE-PACKAGE) that PACKAGE-NAME
 ;;; returns NIL (not an error) for a deleted package, so this is a special
@@ -932,8 +934,8 @@ implementation it is ~S." *default-package-use-list*)
                names))
     res))
 
-(macrolet ((find/intern (function)
-             ;; Both FIND-SYMBOL* and INTERN* require a SIMPLE-STRING,
+(macrolet ((find/intern (function &rest more-args)
+             ;; Both %FIND-SYMBOL and %INTERN require a SIMPLE-STRING,
              ;; but accept a LENGTH. Given a non-simple string,
              ;; we need copy it only if the cumulative displacement
              ;; into the underlying simple-string is nonzero.
@@ -959,13 +961,14 @@ implementation it is ~S." *default-package-use-list*)
                 (truly-the
                  (values symbol (member :internal :external :inherited nil))
                  (,function name length
-                            (find-undeleted-package-or-lose package))))))
+                            (find-undeleted-package-or-lose package)
+                            ,@more-args)))))
 
   (defun intern (name &optional (package (sane-package)))
   #!+sb-doc
   "Return a symbol in PACKAGE having the specified NAME, creating it
   if necessary."
-    (find/intern intern*))
+    (find/intern %intern t))
 
   (defun find-symbol (name &optional (package (sane-package)))
   #!+sb-doc
@@ -973,13 +976,13 @@ implementation it is ~S." *default-package-use-list*)
   then the second value is :INTERNAL, :EXTERNAL or :INHERITED to indicate
   how the symbol is accessible. If no symbol is found then both values
   are NIL."
-    (find/intern find-symbol*)))
+    (find/intern %find-symbol)))
 
 ;;; If the symbol named by the first LENGTH characters of NAME doesn't exist,
 ;;; then create it, special-casing the keyword package.
-(defun intern* (name length package &key no-copy)
+(defun %intern (name length package copy-p)
   (declare (simple-string name) (index length))
-  (multiple-value-bind (symbol where) (find-symbol* name length package)
+  (multiple-value-bind (symbol where) (%find-symbol name length package)
     (cond (where
            (values symbol where))
           (t
@@ -988,10 +991,10 @@ implementation it is ~S." *default-package-use-list*)
            ;; COND, but in case another thread is interning in
            ;; parallel we need to check after grabbing the lock.
            (with-package-graph ()
-             (setf (values symbol where) (find-symbol* name length package))
+             (setf (values symbol where) (%find-symbol name length package))
              (if where
                  (values symbol where)
-                 (let ((symbol-name (cond (no-copy
+                 (let ((symbol-name (cond ((not copy-p)
                                            (aver (= (length name) length))
                                            name)
                                           ((typep name '(simple-array nil (*)))
@@ -1019,7 +1022,7 @@ implementation it is ~S." *default-package-use-list*)
 
 ;;; Check internal and external symbols, then scan down the list
 ;;; of hashtables for inherited symbols.
-(defun find-symbol* (string length package)
+(defun %find-symbol (string length package)
   (declare (simple-string string)
            (type index length))
   (let* ((hash (compute-symbol-hash string length))
@@ -1027,10 +1030,10 @@ implementation it is ~S." *default-package-use-list*)
     (declare (type hash hash ehash))
     (with-symbol ((symbol) (package-internal-symbols package)
                   string length hash ehash)
-      (return-from find-symbol* (values symbol :internal)))
+      (return-from %find-symbol (values symbol :internal)))
     (with-symbol ((symbol) (package-external-symbols package)
                   string length hash ehash)
-      (return-from find-symbol* (values symbol :external)))
+      (return-from %find-symbol (values symbol :external)))
     (let* ((tables (package-tables package))
            (n (length tables)))
       (unless (eql n 0)
@@ -1044,7 +1047,7 @@ implementation it is ~S." *default-package-use-list*)
                                              (svref tables i))
                            string length hash ehash)
                (setf (package-mru-table-index package) i)
-               (return-from find-symbol* (values symbol :inherited)))
+               (return-from %find-symbol (values symbol :inherited)))
              (if (< (decf i) 0) (setq i (1- n)))
              (if (= i start) (return)))))))
   (values nil nil))
@@ -1244,9 +1247,7 @@ uninterned."
                                thing))))
 
 (defun string-listify (thing)
-  (mapcar #'string (if (listp thing)
-                       thing
-                       (list thing))))
+  (mapcar #'string (ensure-list thing)))
 
 (defun export (symbols &optional (package (sane-package)))
   #!+sb-doc
@@ -1629,3 +1630,87 @@ PACKAGE."
   ;; "COMMON-LISP-USER") once and for all here, instead of setting it
   ;; once here and resetting it later.
   (setq *package* *cl-package*))
+
+;;; support for WITH-PACKAGE-ITERATOR
+
+(defun package-iter-init (access-types pkg-designator-list)
+  (declare (type (integer 1 7) access-types)) ; a nonzero bitmask over types
+  (values (logior (ash access-types 3) #b11) 0 #()
+          (package-listify pkg-designator-list)))
+
+;; The STATE parameter is comprised of 4 packed fields
+;;  [0:1] = substate {0=internal,1=external,2=inherited,3=initial}
+;;  [2]   = package with inherited symbols has shadowing symbols
+;;  [3:5] = enabling bits for {internal,external,inherited}
+;;  [6:]  = index into 'package-tables'
+;;
+(defconstant +package-iter-check-shadows+  #b000100)
+
+(defun package-iter-step (start-state index sym-vec pkglist)
+  ;; the defknown isn't enough
+  (declare (type fixnum start-state) (type index index)
+           (type simple-vector sym-vec) (type list pkglist))
+  (declare (optimize speed))
+  (labels
+      ((advance (state) ; STATE is the one just completed
+         (case (logand state #b11)
+           ;; Test :INHERITED first because the state repeats for a package
+           ;; as many times as there are packages it uses. There are enough
+           ;; bits to count up to 2^23 packages if fixnums are 30 bits.
+           (2
+            (when (desired-state-p 2)
+              (let* ((tables (package-tables (this-package)))
+                     (next-state (the fixnum (+ state (ash 1 6))))
+                     (table-idx (ash next-state -6)))
+              (when (< table-idx (length tables))
+                (return-from advance ; remain in state 2
+                  (start next-state (svref tables table-idx))))))
+            (pop pkglist)
+            (advance 3)) ; start on next package
+           (1 ; finished externals, switch to inherited if desired
+            (when (desired-state-p 2)
+              (let ((tables (package-tables (this-package))))
+                (when (plusp (length tables)) ; inherited symbols
+                  (return-from advance ; enter state 2
+                    (start (if (package-%shadowing-symbols (this-package))
+                               (logior 2 +package-iter-check-shadows+) 2)
+                           (svref tables 0))))))
+            (advance 2)) ; skip state 2
+           (0 ; finished internals, switch to externals if desired
+            (if (desired-state-p 1) ; enter state 1
+                (start 1 (package-external-symbols (this-package)))
+                (advance 1))) ; skip state 1
+           (t ; initial state
+            (cond ((endp pkglist) ; latch into returning NIL forever more
+                   (values 0 0 #() '() nil nil))
+                  ((desired-state-p 0) ; enter state 0
+                   (start 0 (package-internal-symbols (this-package))))
+                  (t (advance 0)))))) ; skip state 0
+       (desired-state-p (target-state)
+         (logtest start-state (ash 1 (+ target-state 3))))
+       (this-package ()
+         (truly-the package (car pkglist)))
+       (start (next-state new-table)
+         (let ((symbols (package-hashtable-cells new-table)))
+           (package-iter-step (logior (mask-field (byte 3 3) start-state)
+                                      next-state)
+                              ;; assert that physical length was nonzero
+                              (the index (1- (length symbols)))
+                              symbols pkglist))))
+    (declare (inline desired-state-p this-package))
+    (if (zerop index)
+        (advance start-state)
+        (macrolet ((scan (&optional (guard t))
+                   `(loop
+                     (let ((sym (aref sym-vec (decf index))))
+                       (when (and (pkg-symbol-valid-p sym) ,guard)
+                         (return (values start-state index sym-vec pkglist sym
+                                         (aref #(:internal :external :inherited)
+                                               (logand start-state 3))))))
+                     (when (zerop index)
+                       (return (advance start-state))))))
+          (declare #-sb-xc-host(optimize (sb!c::insert-array-bounds-checks 0)))
+          (if (logtest start-state +package-iter-check-shadows+)
+              (let ((shadows (package-%shadowing-symbols (this-package))))
+                (scan (not (member sym shadows :test #'string=))))
+              (scan))))))
