@@ -65,6 +65,8 @@
 
 ;;;; compiling and loading more of the system
 
+(load "src/cold/muffler.lisp")
+
 ;;; FIXME: CMU CL's pclcom.lisp had extra optional stuff wrapped around
 ;;; COMPILE-PCL, at least some of which we should probably have too:
 ;;;
@@ -107,7 +109,7 @@
                 "SRC;PCL;MACROS"
                 "SRC;PCL;COMPILER-SUPPORT"
                 "SRC;PCL;LOW"
-                "SRC;PCL;SLOT-NAME"
+                #+nil "SRC;PCL;SLOT-NAME" ; moved to build-order.lisp-expr
                 "SRC;PCL;DEFCLASS"
                 "SRC;PCL;DEFS"
                 "SRC;PCL;FNGEN"
@@ -162,8 +164,12 @@
                 "SRC;CODE;STEP"
                 "SRC;CODE;WARM-LIB"
                 #+win32 "SRC;CODE;WARM-MSWIN"
-                "SRC;CODE;RUN-PROGRAM")))
+                "SRC;CODE;RUN-PROGRAM"))
+      (sb-c::*handled-conditions* sb-c::*handled-conditions*))
  (declare (special *compile-files-p*))
+ (proclaim '(sb-ext:muffle-conditions
+             (or (satisfies unable-to-optimize-note-p)
+                 (satisfies optional+key-style-warning-p))))
  (flet
     ((do-srcs (list)
        (dolist (stem list)
