@@ -65,9 +65,10 @@
 
 ;;;; cached functions
 ;;;;
-;;;; FIXME: Is it important to cache these? For performance? Or why?
-;;;; If performance: *Really*? How fast does disassembly need to be??
-;;;; So: Could we just punt this?
+;;;; There's no need for 1000 different versions of a function equivalent
+;;;; to (PROGN (PRINT ADDR) (PRINT OPCODE) (PRINT ARG)) so we try to
+;;;; coalesce sexprs, since there is no such thing as coalescing compiled code.
+;;;; This is not really a "cache" as much as hashtable for coalescing.
 
 (defstruct (fun-cache (:copier nil)
                       (:print-object (lambda (self stream)
@@ -143,6 +144,7 @@
 (defmacro make-dchunk (value)
   `(the dchunk ,value))
 
+#-sb-xc-host ;; FIXME: function belongs in 'target-disassem'
 (defun sap-ref-dchunk (sap byte-offset byte-order)
   (declare (type sb!sys:system-area-pointer sap)
            (type offset byte-offset)
@@ -1544,7 +1546,11 @@
   ;; a sap pointing to our segment
   (segment-sap nil :type (or null sb!sys:system-area-pointer))
   ;; the current segment
-  (segment nil :type (or null segment))
+  ;; But this makes no sense: SEGMENT is defined in 'target-disassem'
+  ;; which means the host can't use an instance of DISASSEM-STATE.
+  ;; Why is this structure defined at all?
+  ;; Perhaps the entire disassembler should be target-only.
+  (segment nil :type (or null #-sb-xc-host segment))
   ;; what to align to in most cases
   (alignment sb!vm:n-word-bytes :type alignment)
   (byte-order :little-endian
