@@ -71,7 +71,7 @@
 (defknown (null symbolp atom consp listp numberp integerp rationalp floatp
                 complexp characterp stringp bit-vector-p vectorp
                 simple-vector-p simple-string-p simple-bit-vector-p arrayp
-                sb!xc:packagep functionp compiled-function-p not)
+                packagep functionp compiled-function-p not)
   (t) boolean (movable foldable flushable))
 
 (defknown (eq eql %eql/integer) (t t) boolean
@@ -175,7 +175,7 @@
 (defknown %make-symbol (simple-string) symbol (flushable))
 (defknown copy-symbol (symbol &optional t) symbol (flushable))
 (defknown gensym (&optional (or string unsigned-byte)) symbol ())
-(defknown symbol-package (symbol) (or sb!xc:package null) (flushable))
+(defknown symbol-package (symbol) (or package null) (flushable))
 (defknown keywordp (t) boolean (flushable))       ; If someone uninterns it...
 
 ;;;; from the "Packages" chapter:
@@ -188,16 +188,16 @@
                                           ;; ### extensions...
                                           (:internal-symbols index)
                                           (:external-symbols index))
-  sb!xc:package)
-(defknown find-package (package-designator) (or sb!xc:package null)
+  package)
+(defknown find-package (package-designator) (or package null)
   (flushable))
 (defknown find-undeleted-package-or-lose (package-designator)
-  sb!xc:package) ; not flushable
+  package) ; not flushable
 (defknown package-name (package-designator) (or simple-string null)
   (unsafely-flushable))
 (defknown package-nicknames (package-designator) list (unsafely-flushable))
 (defknown rename-package (package-designator package-designator &optional list)
-  sb!xc:package)
+  package)
 (defknown package-use-list (package-designator) list (unsafely-flushable))
 (defknown package-used-by-list (package-designator) list (unsafely-flushable))
 (defknown package-shadowing-symbols (package-designator) list (unsafely-flushable))
@@ -529,6 +529,15 @@
   :derive-type #'result-type-first-arg
   :destroyed-constant-args (nth-constant-nonempty-sequence-args 1))
 
+(defknown #.(loop for info across sb!vm:*specialized-array-element-type-properties*
+                  collect
+                  (intern (concatenate 'string "VECTOR-MAP-INTO/"
+                                       (string (sb!vm:saetp-primitive-type-name info)))
+                          :sb!impl))
+    (simple-array index index function list)
+    index
+    (call explicit-check))
+
 ;;; returns the result from the predicate...
 (defknown some (callable sequence &rest sequence) t
   (foldable unsafely-flushable call))
@@ -1197,9 +1206,8 @@
   (any explicit-check)
   :derive-type #'result-type-first-arg)
 
-(defknown output-object (t stream)
-  null
-  (any explicit-check))
+(defknown output-object (t stream) null (any explicit-check))
+(defknown %write (t stream-designator) t (any explicit-check))
 
 (defknown (pprint) (t &optional stream-designator) (values)
   (explicit-check))
@@ -1710,8 +1718,6 @@
 (defknown %check-vector-sequence-bounds (vector index sequence-end)
   index
   (unwind))
-
-(defknown arg-count-error (t t t t t t) nil ())
 
 ;;;; SETF inverses
 
@@ -1796,6 +1802,9 @@
 (defknown sc-number-or-lose (symbol) sc-number
   (foldable))
 
+(defknown set-info-value (t info-number t) t ()
+  :derive-type #'result-type-last-arg)
+
 ;;;; memory barriers
 
 (defknown sb!vm:%compiler-barrier () (values) ())
