@@ -20,6 +20,7 @@
 (def!constant lowtag-limit (ash 1 n-lowtag-bits))
 ;;; the number of tag bits used for a fixnum
 (def!constant n-fixnum-tag-bits
+  (locally (declare (notinline =)) ; avoid unreachable code warning
     (if (= 64 n-word-bits)
         ;; On 64-bit targets, this may be as low as 1 (for 63-bit
         ;; fixnums) and as high as 3 (for 61-bit fixnums).  The
@@ -36,7 +37,7 @@
         ;; constraint on the low end is simple overcrowding of the
         ;; lowtag space, and the constraint on the high end is that it
         ;; must not exceed WORD-SHIFT.
-        (1- n-lowtag-bits)))
+        (1- n-lowtag-bits))))
 ;;; the fixnum tag mask
 (def!constant fixnum-tag-mask (1- (ash 1 n-fixnum-tag-bits)))
 ;;; the bit width of fixnums
@@ -85,6 +86,7 @@
     (let ((stop (1- (ash 1 n-word-bits)))
           (start dynamic-space-start))
       (dolist (other-start (list read-only-space-start static-space-start linkage-table-space-start))
+        (declare (notinline <)) ; avoid dead code note
         (when (< start other-start)
           (setf stop (min stop other-start))))
       stop))
@@ -93,3 +95,9 @@
 ;; a slot of data that is not the instance-layout.
 ;; To get a layout, you must call %INSTANCE-LAYOUT - don't assume index 0.
 (def!constant instance-data-start 1)
+
+;;; Is X a fixnum in the target Lisp?
+#+sb-xc-host
+(defun fixnump (x)
+  (and (integerp x)
+       (<= sb!xc:most-negative-fixnum x sb!xc:most-positive-fixnum)))
