@@ -373,6 +373,9 @@
   #+sb-xc-host nil
   #-sb-xc-host (lambda (name) (if (fboundp name) :function nil)))
 
+;;; Indicates whether the function is deprecated.
+(define-info-type (:function :deprecated) :type-spec deprecation-info)
+
 (declaim (ftype (sfunction (t) ctype)
                 specifier-type ctype-of sb!kernel::ctype-of-array))
 
@@ -460,6 +463,14 @@
 ;;; structure containing the info used to special-case compilation.
 (define-info-type (:function :info) :type-spec (or fun-info null))
 
+;;; This is a type specifier <t> such that if an argument X to the function
+;;; does not satisfy (TYPEP x <t>) then the function definitely returns NIL.
+;;; When the named function is a predicate that appears in (SATISFIES p)
+;;; specifiers, it is possible for type operations to see into the predicate
+;;; just enough to determine that something like
+;;;   (AND (SATISFIES UNINTERESTING-METHOD-REDEFINITION-P) RATIONAL)
+;;; is *empty-type*, which in turn avoids type cache pollution.
+(define-info-type (:function :predicate-truth-constraint) :type-spec t)
 
 ;;;; ":VARIABLE" subsection - Data pertaining to globally known variables.
 
@@ -474,7 +485,7 @@
 (define-info-type (:variable :always-bound)
   :type-spec (member nil :eventually :always-bound))
 
-(define-info-type (:variable :deprecated) :type-spec t)
+(define-info-type (:variable :deprecated) :type-spec deprecation-info)
 
 ;;; the declared type for this variable
 (define-info-type (:variable :type)
@@ -634,6 +645,9 @@
 
 (define-info-type (:type :source-location) :type-spec t)
 
+;;; Indicates whether the function is deprecated.
+(define-info-type (:type :deprecated) :type-spec deprecation-info)
+
 ;;;; ":TYPED-STRUCTURE" subsection.
 ;;;; Data pertaining to structures that used DEFSTRUCT's :TYPE option.
 (define-info-type (:typed-structure :info) :type-spec t)
