@@ -65,6 +65,9 @@
   ;; keys ... data ...
   )
 
+(declaim (ftype (sfunction (unsigned-byte)
+                           (unsigned-byte #.sb!vm:n-positive-fixnum-bits))
+                primify))
 (defun make-info-storage (n-cells-min &optional (load-factor .7))
   ;; If you ask for 40 entries at 50% load, you get (PRIMIFY 80) entries.
   (let* ((n-cells (primify (ceiling n-cells-min load-factor)))
@@ -79,6 +82,8 @@
     (fill a nil :start end)
     a))
 
+(declaim (ftype (sfunction (t) (unsigned-byte #.sb!vm:n-positive-fixnum-bits))
+                globaldb-sxhashoid))
 (defstruct (info-hashtable (:conc-name info-env-))
   (storage (make-info-storage 30) :type simple-vector)
   (comparator #'equal :type function)
@@ -98,10 +103,7 @@
 ;; or info-puthash, but that's fine because there's only one thread.
 (defmacro info-cas (storage index oldval newval)
   #+sb-xc-host
-  `(let ((actual-old (svref ,storage ,index)))
-     (if (eq actual-old ,oldval)
-         (progn (setf (svref ,storage ,index) ,newval) actual-old)
-         (error "xc bug. CAS expected ~S got ~S" actual-old ,oldval)))
+  `(xc-compare-and-swap-svref ,storage ,index ,oldval ,newval)
   #-sb-xc-host
   `(cas (svref ,storage ,index) ,oldval ,newval))
 
@@ -127,6 +129,11 @@
 ;; never present as a value.
 #+sb-xc-host
 (progn
+  (defun xc-compare-and-swap-svref (vector index old new)
+    (let ((actual-old (svref vector index)))
+      (if (eq old actual-old)
+          (progn (setf (svref vector index) new) old)
+          (error "xc bug. CAS expected ~S got ~S" old actual-old))))
   (defun make-info-forwarding-pointer (index) index)
   (defun info-forwarding-pointer-target (pointer) pointer)
   (defun info-value-moved-p (val) (fixnump val)))
@@ -159,8 +166,8 @@
 ;; from scanning physically forward, preferably within a cache line,
 ;; but for practical purposes linear probing is worse.]
 ;;
-(defmacro do-probe-sequence ((storage key table &optional hash)
-                             &key probe hit miss)
+(defmacro !do-probe-sequence ((storage key table &optional hash)
+                              &key probe hit miss)
   (with-unique-names (test miss-fn len key-index step)
     (once-only ((storage storage) (key key) (table table)
                 (hashval
@@ -227,7 +234,7 @@
 ;; or started all over again before the reader got a chance to chase one time.
 ;;
 (defun info-gethash (key env &aux (storage (info-env-storage env)))
-  (do-probe-sequence (storage key env)
+  (!do-probe-sequence (storage key env)
     :miss (return-from info-gethash nil)
     ;; With 99% certainty the :READ barrier is needed for non-x86, and if not,
     ;; it can't hurt. 'info-storage-next' can be empty until at least one cell
@@ -288,7 +295,7 @@
                               (update array index actual-old)))))))))
     (named-let probe ((hashval (funcall (info-env-hash-function env) key))
                       (storage (info-env-storage env)))
-      (do-probe-sequence (storage key env hashval)
+      (!do-probe-sequence (storage key env hashval)
        :hit (follow/update storage (value-index))
        :miss
        (progn
@@ -348,7 +355,7 @@
           unless (eql key +empty-key+)
           do (let* ((new-key-index
                       (block nil
-                        (do-probe-sequence (new-storage key env)
+                        (!do-probe-sequence (new-storage key env)
                           :hit (bug "Globaldb rehash failure. Mutated key?")
                           :miss (return (key-index)))))
                     (old-value-index (+ old-key-index old-capacity))
@@ -490,10 +497,6 @@
 ;; Descriptors are target fixnums
 (deftype info-descriptor () `(signed-byte ,sb!vm:n-fixnum-bits))
 
-;; Every Name amenable to storage in info-vectors has an auxilliary key
-;; as explained above, except that the root name itself has none.
-(defconstant +no-auxilliary-key+ 0)
-
 ;; An empty info-vector. Its 0th field describes that there are no more fields.
 (defconstant-eqx +nil-packed-infos+ #(0) #'equalp)
 
@@ -506,17 +509,9 @@
 ;; A field is either a count of info-numbers, or an info-number.
 (declaim (inline packed-info-field))
 (defun packed-info-field (vector desc-index field-index)
-  ;; Should not need (THE INFO-NUMBER) however type inference
-  ;; seems borked during cross-compilation due to the shadowed LDB
-  ;; (see "don't watch:" in cold/defun-load-or-cload-xcompiler)
-  ;; and in particular it sees especially weird that this message appears
-  ;;    note: type assertion too complex to check:
-  ;;    (VALUES (UNSIGNED-BYTE 6) &REST T).
-  ;; because nothing here should be possibly-multiple-value-producing.
-  (the info-number
-    (ldb (byte info-number-bits
-               (* (the (mod #.+infos-per-word+) field-index) info-number-bits))
-         (the info-descriptor (svref vector desc-index)))))
+  (ldb (byte info-number-bits
+             (* (the (mod #.+infos-per-word+) field-index) info-number-bits))
+       (the info-descriptor (svref vector desc-index))))
 
 ;; Compute the number of elements needed to hold unpacked VECTOR after packing.
 ;; This is not "compute-packed-info-size" since that could be misconstrued
@@ -596,9 +591,9 @@
 ;; returns the next field from a descriptor in INPUT-VAR, a packed vector.
 ;; The generator uses DESCRIPTOR-INDEX and updates it as a side-effect.
 ;;
-(defmacro with-packed-info-iterator ((generator input-var
-                                                &key descriptor-index)
-                                     &body body)
+(defmacro !with-packed-info-iterator ((generator input-var
+                                                 &key descriptor-index)
+                                      &body body)
   (with-unique-names (input word count)
     `(let* ((,input (the simple-vector ,input-var))
             (,descriptor-index -1)
@@ -620,8 +615,8 @@
 ;; Iterate over VECTOR, binding DATA-INDEX to the index of each aux-key in turn.
 ;; TOTAL-N-FIELDS is deliberately exposed to invoking code.
 ;;
-(defmacro do-packed-info-vector-aux-key ((vector &optional (data-index (gensym)))
-                                         step-form &optional result-form)
+(defmacro !do-packed-info-vector-aux-key ((vector &optional (data-index (gensym)))
+                                          step-form &optional result-form)
   (with-unique-names (descriptor-idx field-idx)
     (once-only ((vector vector))
       `(let ((,data-index (length ,vector))
@@ -644,6 +639,21 @@
                  (return ,result-form))
                ,@(if step-form (list step-form)))))))
 
+;; Return all function names that are stored in SYMBOL's info-vector.
+;; As an example, (INFO-VECTOR-NAME-LIST 'SB-PCL::DIRECT-SUPERCLASSES) =>
+;; ((SB-PCL::SLOT-ACCESSOR :GLOBAL SB-PCL::DIRECT-SUPERCLASSES SB-PCL::READER)
+;;  (SB-PCL::SLOT-ACCESSOR :GLOBAL SB-PCL::DIRECT-SUPERCLASSES BOUNDP)
+;;  (SB-PCL::SLOT-ACCESSOR :GLOBAL SB-PCL::DIRECT-SUPERCLASSES SB-PCL::WRITER))
+(defun info-vector-name-list (symbol)
+  (let ((vector (symbol-info-vector symbol))
+        (list))
+    (when vector
+      (!do-packed-info-vector-aux-key (vector key-index)
+        (push (construct-globaldb-name (svref vector key-index) symbol)
+              list))
+      (nconc (and (plusp (packed-info-field vector 0 0)) (list symbol))
+             (nreverse list)))))
+
 ;; Compute the number of elements needed to hold packed VECTOR after unpacking.
 ;; The unpacked size is the number of auxilliary keys plus the number of entries
 ;; @ 2 cells per entry, plus the number of length cells which indicate the
@@ -652,7 +662,7 @@
 ;;
 (defun compute-unpackified-info-size (vector)
   (declare (simple-vector vector))
-  (do-packed-info-vector-aux-key (vector) ()
+  (!do-packed-info-vector-aux-key (vector) ()
     ;; off-by-one: the first info group's auxilliary key is imaginary
     (1- (truly-the fixnum (ash total-n-fields 1)))))
 
@@ -666,7 +676,7 @@
     (setq output (make-array (compute-unpackified-info-size input))))
   (let ((i (length input)) (j -1))  ; input index and output index respectively
     (declare (type index-or-minus-1 i j))
-    (with-packed-info-iterator (next-field input :descriptor-index desc-idx)
+    (!with-packed-info-iterator (next-field input :descriptor-index desc-idx)
       (loop ; over name
          (let ((n-infos (next-field)))
            ;; store the info group length, including the length cell in the length
@@ -851,10 +861,6 @@
 ;; Search packed VECTOR for AUX-KEY and INFO-NUMBER, returning
 ;; the index of the data if found, or NIL if not found.
 ;;
-(declaim (ftype (function (simple-vector (or (eql 0) symbol) info-number)
-                          (or null index))
-                packed-info-value-index))
-
 (defun packed-info-value-index (vector aux-key type-num)
   (declare (optimize (safety 0))) ; vector bounds are AVERed
   (let ((data-idx (length vector)) (descriptor-idx 0) (field-idx 0))
@@ -996,7 +1002,7 @@
   (let ((name root-symbol)
         (data-idx (length vect)))
     (declare (type index data-idx))
-    (with-packed-info-iterator (next-field vect :descriptor-index desc-idx)
+    (!with-packed-info-iterator (next-field vect :descriptor-index desc-idx)
       (loop ; over name
          (dotimes (i (next-field)) ; number of infos for this name
            (funcall function name (next-field) (svref vect (decf data-idx))))
@@ -1109,20 +1115,16 @@ This is interpreted as
 ;;; Some of this stuff might belong in 'symbol.lisp', but can't be,
 ;;; because 'symbol.lisp' is :NOT-HOST in build-order.
 
+;; In the target, UPDATE-SYMBOL-INFO is defined in 'symbol.lisp'.
 #+sb-xc-host
-;; SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
-;; and UPDATE-SYMBOL-INFO is defined in 'symbol.lisp'.
-;; But in the host Lisp, there is no such thing as a symbol-info slot,
-;; even if the host is SBCL. Instead, symbol-info is kept in the symbol-plist.
-(macrolet ((get-it () '(get symbol :sb-xc-globaldb-info)))
-  (defun symbol-info (symbol) (get-it))
-  (defun update-symbol-info (symbol update-fn)
-    ;; Never pass NIL to an update-fn. Pass the minimal info-vector instead,
-    ;; a vector describing 0 infos and 0 auxilliary keys.
-    (let ((newval (funcall update-fn (or (get-it) +nil-packed-infos+))))
-      (when newval
-        (setf (get-it) newval))
-      (values))))
+(defun update-symbol-info (symbol update-fn)
+  ;; Never pass NIL to an update-fn. Pass the minimal info-vector instead,
+  ;; a vector describing 0 infos and 0 auxilliary keys.
+  (let ((newval (funcall update-fn (or (symbol-info-vector symbol)
+                                       +nil-packed-infos+))))
+    (when newval
+      (setf (symbol-info-vector symbol) newval))
+    (values)))
 
 ;; Return the globaldb info for SYMBOL. With respect to the state diagram
 ;; presented at the definition of SYMBOL-PLIST, if the object in SYMBOL's
@@ -1131,15 +1133,21 @@ This is interpreted as
 ;; In terms of this function being named "-vector", implying always a vector,
 ;; it is understood that NIL is a proxy for +NIL-PACKED-INFOS+, a vector.
 ;;
-#!-symbol-info-vops (declaim (inline symbol-info-vector))
-(defun symbol-info-vector (symbol)
+#-sb-xc-host
+(progn
+ #!-symbol-info-vops (declaim (inline symbol-info-vector))
+ (defun symbol-info-vector (symbol)
   (let ((info-holder (symbol-info symbol)))
     (truly-the (or null simple-vector)
-               (if (listp info-holder) (cdr info-holder) info-holder))))
+               (if (listp info-holder) (cdr info-holder) info-holder)))))
 
 ;;; The current *INFO-ENVIRONMENT*, a structure of type INFO-HASHTABLE.
-(declaim (type info-hashtable *info-environment*))
-(defvar *info-environment*)
+;;; Cheat by setting to nil before the type is proclaimed
+;;; so that we can then also proclaim ALWAYS-BOUND.
+(defvar *info-environment* nil)
+#-sb-xc-host
+(declaim (type info-hashtable *info-environment*)
+         (always-bound *info-environment*))
 
 ;;; Update the INFO-NUMBER for NAME in the global environment,
 ;;; setting it to NEW-VALUE. This is thread-safe in the presence
