@@ -78,9 +78,6 @@
 ;;; This is possibly surprising to people since there seems to be some
 ;;; expectation that a DEFSETF lambda list is a macro lambda list,
 ;;; which it isn't. We'll relax and accept &ENVIRONMENT in the middle.
-;;; But we won't accept the ugly syntax that parse-defmacro accidentally
-;;; allows of (A B &ENVIRONMENT E X Y) which has 4 positional parameters.
-;;; Nor can it appear between &KEY and &ALLOW-OTHER-KEYS.
 ;;;
 (defun parse-lambda-list
     (list &key (context "an ordinary lambda list")
@@ -105,6 +102,15 @@
                           collect `(,i (setq ,s ,val))))))
     (labels ((destructuring-p ()
                (logtest (bits &whole) accept))
+             (probably-ll-keyword-p (arg)
+               ;; Compiler doesn't see that the check is manually done. :-(
+               #-sb-xc-host
+               (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+               (and (symbolp arg)
+                    (let ((name (symbol-name arg)))
+                      (and (typep name 'simple-base-string)
+                           (plusp (length name))
+                           (char= (char name 0) #\&)))))
              (need-arg (state)
                (croak "expecting variable after ~A in: ~S" state list))
              (need-symbol (x why)
@@ -116,16 +122,27 @@
                  (if (destructuring-p)
                      (croak "~A is not a symbol or list: ~S" why x)
                      (croak "~A is not a symbol: ~S" why x))))
+             (defaultp (x what-kind)
+               (cond ((symbolp x) nil)
+                     ((listp x) t)
+                     (t (croak "~A parameter is not a symbol or cons: ~S"
+                               what-kind x))))
              (croak (string &optional (a1 0 a1p) (a2 0 a2p) (a3 0 a3p))
                ;; Don't care that FUNCALL can't elide fdefinition here.
                (declare (optimize (speed 1)))
                (let ((l (if a1p (list a1 a2 a3))))
                  (if (and l (not a3p)) (rplacd (if a2p (cdr l) l) nil))
                  ;; KLUDGE: When this function was limited to parsing
-                 ;; ordinary lambda lists, this error call was COMPILER-ERROR.
-                 ;; To make all tests pass, it has to decide what to be.
-                 ;; It's possible the tests are poorly designed and are
-                 ;; acting as "change detectors"
+                 ;; ordinary lambda lists, this error call was always
+                 ;; COMPILER-ERROR, which must be used, not plain old ERROR,
+                 ;; to avoid the compiler itself crashing. But outside of
+                 ;; the compiler, it must be ERROR. This discrepancy is sad
+                 ;; since DESTRUCTURING-BIND herein can cause a compiler crash.
+                 ;; It seems that the right thing is for the compiler to wrap
+                 ;; a condition handler around PARSE-LAMBDA-LIST.
+                 ;; Expecting a callee to understand how to signal conditions
+                 ;; tailored to a particular caller is not how things are
+                 ;; supposed to work.
                  (funcall (if (destructuring-p) 'error 'compiler-error)
                           condition-class
                           :format-control string :format-arguments l))))
@@ -150,9 +167,7 @@
            (return))
          (shiftf last-arg arg (pop input))
 
-         (when (and (symbolp arg)
-                    (let ((name (symbol-name arg)))
-                      (and (plusp (length name)) (char= (char name 0) #\&))))
+         (when (probably-ll-keyword-p arg)
            ;; Handle a probable lambda list keyword
            (multiple-value-bind (from-states to-state)
               (case arg
@@ -262,29 +277,42 @@
         (dolist (arg required)
           (need-bindable arg "Required argument"))
         ;; FIXME: why not check symbol-ness of supplied-p variables now?
-        (flet ((defaultp (x what-kind)
-                 (cond ((symbolp x) nil)
-                       ((listp x) t)
-                       (t (croak "~A parameter is not a symbol or cons: ~S"
-                                 what-kind x)))))
-          (dolist (arg optional)
-            (when (defaultp arg '&optional)
-              (destructuring-bind (var &optional init-form supplied-p) arg
-                (declare (ignore init-form supplied-p))
-                (need-bindable var "&OPTIONAL parameter name"))))
+        (flet ((scan-opt/key (list what-kind description)
+                 (dolist (arg list)
+                   (when (defaultp arg what-kind)
+                     ;; FIXME:  (DEFUN F (&OPTIONAL (A B C D)) 42) crashes the
+                     ;; compiler, but not as consequence of the new parser.
+                     ;; (This is not a regression)
+                     (destructuring-bind (var &optional default sup-p) arg
+                       (if (and (consp var) (eq what-kind '&key))
+                           (destructuring-bind (keyword-name var) var
+                             (declare (ignore keyword-name))
+                             (need-bindable var description))
+                           (need-bindable var description))
+                       ;; Inform the user about a possibly malformed
+                       ;; destructuring list (&OPTIONAL (A &OPTIONAL B)).
+                       ;; It's technically legal but unlikely to be right,
+                       ;; as A's default form is the symbol &OPTIONAL,
+                       ;; which is an unlikely name for a local variable,
+                       ;; and an illegal name for a DEFVAR or such,
+                       ;; being in the CL package.
+                       (unless silent
+                         (when (and (probably-ll-keyword-p default)
+                                    (member default sb!xc:lambda-list-keywords))
+                           (style-warn "suspicious default ~S in lambda list: ~S."
+                                       default list))
+                         (when (and (probably-ll-keyword-p sup-p)
+                                    (member sup-p sb!xc:lambda-list-keywords))
+                           (style-warn
+                            "suspicious supplied-p variable ~S in lambda list: ~S."
+                            sup-p list))))))))
+          (scan-opt/key optional '&optional "&OPTIONAL parameter name")
           (when rest
             (need-bindable (car rest) "&REST argument"))
-          (dolist (arg keys)
-            (when (defaultp arg '&key)
-              (destructuring-bind (var-or-kv &optional init-form supplied-p) arg
-                (declare (ignore init-form supplied-p))
-                (if (atom var-or-kv)
-                    (need-symbol var-or-kv "&KEY parameter name")
-                    (destructuring-bind (keyword-name var) var-or-kv
-                      (declare (ignore keyword-name))
-                      (need-bindable var "&KEY parameter name"))))))
+          (scan-opt/key keys '&key "&KEY parameter name")
           (dolist (arg aux)
             (when (defaultp arg '&aux)
+              ;; FIXME: also potentially compiler-crash-inducing
               (destructuring-bind (var &optional init-form) arg
                 (declare (ignore init-form))
                 ;; &AUX is not destructured
@@ -301,13 +329,17 @@
 ;;; - a &REST arg that destructures preceded by any optional args.
 ;;;   It's suspicious because if &REST destructures, then in essence it
 ;;;   must not be NIL, which means the optionals aren't really optional.
-(defun parse-ds-lambda-list (lambda-list &key silent)
+(defun parse-ds-lambda-list (lambda-list
+                             &key silent
+                             (condition-class 'simple-program-error))
   (multiple-value-bind (llks required optional rest keys aux env whole)
       (parse-lambda-list lambda-list
                          :accept (lambda-list-keyword-mask 'destructuring-bind)
-                         :context 'destructuring-bind)
+                         :context 'destructuring-bind
+                         :silent silent :condition-class condition-class)
    (declare (ignore env) (notinline mapcar))
-   (labels ((parse (list) (if (atom list) list (parse-ds-lambda-list list)))
+   (labels ((parse (list)
+              (if (atom list) list (parse-ds-lambda-list list :silent silent)))
             (parse* (list arg-specifier)
               (let ((parse (parse list)))
                 (when (and (not silent) (vectorp parse)) ; is destructuring
@@ -345,10 +377,21 @@
                  when sym collect `(,sym (svref ,parts ,i)))
        ,@body)))
 
-;; Split a keyword argument specifier into the keyword, the bound variable
-;; or destructuring pattern, the default, and supplied-p var. If present the
-;; supplied-p var is in a singleton list.
-;; DEFAULT should be specified as '* when parsing a DEFTYPE lambda-list.
+;;; Split an optional argument specifier into the bound variable
+;;; or destructuring pattern, the default, and supplied-p var.
+;;; If present the supplied-p var is in a singleton list.
+;;; DEFAULT should be specified as '* when parsing a DEFTYPE lambda-list.
+(defun parse-optional-arg-spec (spec &optional default)
+  (etypecase spec
+    (symbol (values spec default nil))
+    (cons (values (car spec)
+                  (if (cdr spec) (cadr spec) default)
+                  (cddr spec)))))
+
+;;; Split a keyword argument specifier into the keyword, the bound variable
+;;; or destructuring pattern, the default, and supplied-p var.
+;;; If present the supplied-p var is in a singleton list.
+;;; DEFAULT should be specified as '* when parsing a DEFTYPE lambda-list.
 (defun parse-key-arg-spec (spec &optional default)
   (etypecase spec
     (symbol (values (keywordicate spec) spec default nil))
@@ -417,17 +460,37 @@
                  ;; Should &AUX be inserted even if empty? Probably not.
                  (if aux (cons '&aux aux)))))))
 
-;; Produce a destructuring lambda list from its internalized representation,
-;; excluding any parts that don't constrain the shape of the expected input.
-;; &AUX, supplied-p vars, and defaults do not impose shape constraints.
+;;; Produce a destructuring lambda list from its internalized representation,
+;;; excluding any parts that don't constrain the shape of the expected input.
+;;; &AUX, supplied-p vars, and defaults do not impose shape constraints.
+;;;
+;;; However as a special case, some constant defaults are retained mainly for
+;;; backward-compatibility.
+;;; The reason for it is that a test in SB-INTROSPECT checks the lambda list
+;;; of type ARRAY, which has explicit '* defaults. They're explicit
+;;; because of a bug or deficiency in !DEF-TYPE-TRANSLATOR which does not
+;;; adhere to the convention that DEFTYPE-like lambda lists use '* as implicit
+;;; defaults for everything unless stated otherwise.
+;;; So really the '* should have been superfluous in the lambda list,
+;;; but I'm also not convinced that changing the test case is the right thing.
+;;;
 (defun unparse-ds-lambda-list (parsed-lambda-list &optional cache)
   (cond ((symbolp parsed-lambda-list) parsed-lambda-list)
         ((cdr (assq parsed-lambda-list (cdr cache))))
         (t
          (with-ds-lambda-list-parts (llks whole req optional rest keys)
              parsed-lambda-list
-           (labels ((remove-default (spec)
-                      (if (atom spec) spec (list (recurse (car spec)))))
+           (labels ((process-opt (spec)
+                      (if (atom spec)
+                          spec
+                          (cons (recurse (car spec)) (maybe-default spec))))
+                    (maybe-default (spec)
+                      (let ((def (cdr spec)))
+                        (when (and def (typep (car def)
+                                              '(or (cons (eql quote))
+                                                   (member t)
+                                                   keyword number)))
+                          (list (car def))))) ; Remove any supplied-p var.
                     (recurse (x) (unparse-ds-lambda-list x cache))
                     (memoize (input output)
                       (when cache (push (cons input output) (cdr cache)))
@@ -439,12 +502,13 @@
                ;; &WHOLE is omitted unless it destructures something.
                (when (vectorp (car whole)) (list (recurse (car whole))))
                (mapcar #'recurse req)
-               (mapcar #'remove-default optional)
+               (mapcar #'process-opt optional)
                (when rest (list (recurse (car rest))))
                (mapcar (lambda (x)
                          (if (typep x '(cons cons))
-                             (list (list (caar x) (recurse (cadar x))))
-                             (remove-default x)))
+                             (cons (list (caar x) (recurse (cadar x)))
+                                   (maybe-default x))
+                             (process-opt x)))
                        keys))))))))
 
 ;;; Return the list of variables bound by a destructuring lambda list.
@@ -464,7 +528,7 @@
                             (t (error "Excess args")))))
           (doer))
 |#
-(defun ds-lambda-list-symbols (parsed-lambda-list)
+(defun ds-lambda-list-variables (parsed-lambda-list &optional (include-aux t))
   (collect ((output))
     (labels ((recurse (x) (if (vectorp x) (scan x) (output x)))
              (copy (x) (dolist (elt x) (recurse elt)))
@@ -483,8 +547,9 @@
                          (t (let ((k (car x)))
                               (if (symbolp k) (output k) (recurse (cadr k)))
                               (suppliedp-var x)))))
-                 (dolist (x aux)
-                   (output (if (symbolp x) x (car x)))))))
+                 (when include-aux
+                   (dolist (x aux)
+                     (output (if (symbolp x) x (car x))))))))
       (recurse parsed-lambda-list)
       (output))))
 
@@ -709,19 +774,14 @@
                ;; Optionals.
                (do ((elts opt (cdr elts)))
                    ((endp elts))
-                 (let ((elt (car elts)))
-                   (multiple-value-bind (var def sup-p-var)
-                       (if (atom elt)
-                           (values elt)
-                           (values (car elt)
-                                   (if (cdr elt) (cadr elt) default-default)
-                                   (cddr elt)))
-                     (bind-if t input
-                              (if explicit-cast
-                                  (cast/pop `(,explicit-cast list ,input)
-                                            (cdr elts))
-                                  `(pop ,input))
-                              var sup-p-var def)))))
+                 (multiple-value-bind (var def sup-p-var)
+                     (parse-optional-arg-spec (car elts) default-default)
+                   (bind-if t input
+                            (if explicit-cast
+                                (cast/pop `(,explicit-cast list ,input)
+                                          (cdr elts))
+                                `(pop ,input))
+                            var sup-p-var def))))
 
              ;; The spec allows the inane use of (A B &REST (C D)) = (A B C D).
              ;; The former is less efficient, since it is "nested", only not.
@@ -747,22 +807,101 @@
 
 ;;; Runtime support
 
-;; Extract the context from a destructuring-bind pattern as represented in
-;; a call to a checking function. A "context" is a non-bindable subpattern
-;; headed by :MACRO (as it came from a macro-like operator, e.g. DEFTYPE).
-;; Return three values: CONTEXT-NAME, CONTEXT-KIND, and the real pattern.
-(defun get-ds-bind-context (pattern)
-  (let ((marker (car pattern)))
-    (if (eq (if (listp marker) (car marker)) :macro)
-        (let ((context (cdr marker)))
-          (values (car context) (cdr context) (cdr pattern)))
-        (values nil 'destructuring-bind pattern))))
-
 ;; Given FORM as the input to a compiler-macro, return the argument forms
 ;; that the called function would receive, skipping over FUNCALL.
 (defun compiler-macro-args (form)
   (cdr (if (eql (car form) 'funcall) (cdr form) form)))
 
+;; Extract the context from a destructuring-bind pattern as represented in
+;; a call to a checking function. A "context" is a non-bindable subpattern
+;; headed by :MACRO (if it came from any macro-like thing, e.g. DEFTYPE),
+;; or :SPECIAL-FORM.
+;; Return three values: CONTEXT-NAME, CONTEXT-KIND, and the real pattern.
+(defun get-ds-bind-context (pattern)
+  (let ((marker (car pattern)))
+    (case (and (listp marker) (car marker))
+      (:macro
+       (let ((context (cdr marker)))
+         (values (car context) (cdr context) (cdr pattern))))
+      (:special-form
+       (values (cdr marker) :special-form (cdr pattern)))
+      (:eval
+       (values nil :eval (cdr pattern)))
+      (t
+       (values nil 'destructuring-bind pattern)))))
+
+;;; Helpers for the variations on CHECK-DS-mumble.
+(defun ds-bind-error (input min max pattern)
+  (multiple-value-bind (name kind lambda-list) (get-ds-bind-context pattern)
+    #-sb-xc-host
+    (declare (optimize sb!c::allow-non-returning-tail-call))
+    (case kind
+     (:special-form
+      ;; IR1 translators should call COMPILER-ERROR instead of
+      ;; ERROR. To ingrain that knowledge into the CHECK-DS-foo
+      ;; functions is a bit of a hack, but to do otherwise
+      ;; changes how DS-BIND has to expand.
+      (compiler-error 'sb!kernel::arg-count-error
+                      :kind "special operator" :name name
+                      :args input :lambda-list lambda-list
+                      :minimum min :maximum max))
+     #!+sb-eval
+     (:eval
+      (error 'sb!eval::arg-count-program-error
+             ;; This is stupid. Maybe we should just say
+             ;;  "error parsing special form"?
+             ;; It would be more sensible than mentioning
+             ;; a random nonstandard macro.
+             :kind 'sb!eval::program-destructuring-bind
+             :args input :lambda-list lambda-list
+             :minimum min :maximum max))
+     (t
+      (error 'sb!kernel::arg-count-error
+             :kind kind :name name
+             :args input :lambda-list lambda-list
+             :minimum min :maximum max)))))
+
+(defun check-ds-bind-keys (input plist valid-keys pattern)
+  ;; Check just the keyword portion of the input in PLIST
+  ;; against VALID-KEYS. If VALID-KEYS = NIL then we don't care what
+  ;; the keys are - &ALLOW-OTHER-KEYS was present in the lambda-list,
+  ;; and we don't care if non-symbols are found in keyword position.
+  ;; Always enforce that the list has even length though.
+  (let* (seen-allowp seen-other bad-key
+         (tail plist)
+         (problem
+          (loop
+           (when (null tail)
+             (if seen-other
+                 (return :unknown-keyword)
+                 (return-from check-ds-bind-keys input)))
+           (unless (listp tail) (return :dotted-list))
+           (let ((next (cdr tail)))
+             (when (null next) (return :odd-length))
+             (unless (listp next) (return :dotted-list))
+             (let ((key (car tail)))
+               (when valid-keys
+                 (if (eq key :allow-other-keys) ; always itself allowed
+                     (unless seen-allowp
+                       (setf seen-allowp t)
+                       (when (car next) ; :allow-other-keys <non-nil>
+                         (setf seen-other nil valid-keys nil)))
+                     (unless (or seen-other
+                                 (find key (truly-the simple-vector valid-keys)
+                                       :test 'eq))
+                       (setq seen-other t bad-key key)))))
+             (setq tail (cdr next))))))
+    (multiple-value-bind (kind name) (get-ds-bind-context pattern)
+      #-sb-xc-host
+      (declare (optimize sb!c::allow-non-returning-tail-call))
+      (error 'sb!kernel::defmacro-lambda-list-broken-key-list-error
+             :kind kind :name name
+             :problem problem
+             :info (if (eq problem :unknown-keyword)
+                       ;; show any one unaccepted keyword
+                       (list bad-key (coerce valid-keys 'list))
+                       plist)))))
+
 (macrolet ((scan-req-opt ((input min max pattern list-name actual-max)
                           &key if-list-exhausted if-max-reached)
              ;; Decide whether the input matches up to the end of
@@ -777,18 +916,15 @@
                 (declare (type index count))
                 (loop (when (zerop count) (return ,if-max-reached))
                       (when (null ,list-name)
-                        (if (< (- max count) ,min)
-                            (min/max-err ,pattern ,actual-max)
-                            (return ,if-list-exhausted)))
+                        (return
+                         (if (< (- max count) ,min)
+                             (ds-bind-error ,input ,min ,actual-max ,pattern)
+                             ,if-list-exhausted)))
                       (unless (listp ,list-name) ; dotted list error
-                        (min/max-err ,pattern ,actual-max))
+                        (return
+                         (ds-bind-error ,input ,min ,actual-max ,pattern)))
                       (decf count)
-                      (setq ,list-name (cdr ,list-name)))))
-           (min/max-err (pattern effective-max)
-             `(multiple-value-bind (name kind lambda-list)
-                  (get-ds-bind-context ,pattern)
-               (sb!kernel::arg-count-error kind name input lambda-list
-                                           min ,effective-max))))
+                      (setq ,list-name (cdr ,list-name))))))
 
   ;; Assert that INPUT has the requisite number of elements as
   ;; specified by MIN/MAX. PATTERN does not contain &REST or &KEY.
@@ -797,8 +933,9 @@
     (scan-req-opt (input min max pattern list max)
                   ;; If 'count' became zero, then since there was
                   ;; no &REST, the LIST had better be NIL.
-                  :if-max-reached (if list (min/max-err pattern max) input)
-                  ;; The loop checks for dotted tail and at least MIN elements,
+                  :if-max-reached
+                  (if list (ds-bind-error input min max pattern) input)
+                  ;; The loop checks for dotted tail and >= MIN elements,
                   ;; so end of list means a valid match to the pattern.
                   :if-list-exhausted input))
 
@@ -809,53 +946,14 @@
     (scan-req-opt (input min max pattern list nil)
                   :if-list-exhausted input :if-max-reached input))
 
-  ;; Check just the keyword portion of the input in PLIST
-  ;; against VALID-KEYS. If VALID-KEYS = NIL then we don't care what
-  ;; the keys are - &ALLOW-OTHER-KEYS was present in the lambda-list,
-  ;; and we don't care if non-symbols are found in keyword position.
-  ;; We always enforce that the list is not odd-length though.
-  (flet ((check-plist (input plist valid-keys pattern)
-           (let ((list plist) seen-allowp seen-other bad-key)
-             (flet ((validp (key)
-                      (find key (truly-the simple-vector valid-keys) :test 'eq))
-                    (croak (problem &optional (info plist))
-                      (multiple-value-bind (kind name)
-                          (get-ds-bind-context pattern)
-                       (error 'sb!kernel::defmacro-lambda-list-broken-key-list-error
-                              :kind kind :name name
-                              :problem problem :info info))))
-               (declare (inline validp))
-               (loop
-                (when (null list)
-                  (if seen-other
-                      (croak :unknown-keyword ; show any one unaccepted keyword
-                             (list bad-key
-                                   (coerce (truly-the simple-vector valid-keys)
-                                           'list)))
-                      (return input)))
-                (unless (listp list) (croak :dotted-list))
-                (let ((next (cdr list)))
-                  (when (null next) (croak :odd-length))
-                  (unless (listp next) (croak :dotted-list))
-                  (let ((key (car list)))
-                    (when valid-keys
-                      (if (eq key :allow-other-keys) ; always itself allowed
-                          (unless seen-allowp
-                            (when (car next) ; :allow-other-keys <non-nil>
-                              (setf valid-keys nil seen-other nil))
-                            (setf seen-allowp t))
-                          (when (and (not seen-other) (not (validp key)))
-                            (setq seen-other t bad-key key)))))
-                  (setq list (cdr next))))))))
-
-    ;; The pattern contains &KEY. Anything beyond the final optional arg
-    ;; must be a well-formed property list regardless of existence of &REST.
-    (defun check-ds-list/&key (input min max pattern valid-keys)
-      (declare (type index min max) (optimize speed))
-      (scan-req-opt (input min max pattern list nil)
-                    :if-list-exhausted input
-                    :if-max-reached (check-plist input list valid-keys
-                                                 pattern)))
+  ;; The pattern contains &KEY. Anything beyond the final optional arg
+  ;; must be a well-formed property list regardless of existence of &REST.
+  (defun check-ds-list/&key (input min max pattern valid-keys)
+    (declare (type index min max) (optimize speed))
+    (scan-req-opt (input min max pattern list nil)
+                  :if-list-exhausted input
+                  :if-max-reached (check-ds-bind-keys
+                                   input list valid-keys pattern)))
 
     ;; Compiler-macro lambda lists are macro lambda lists -- meaning that
     ;; &key ((a a) t) should match a literal A, not a form evaluating to A
@@ -878,28 +976,27 @@
     ;; to validate keywords, a compiler-macro is probably always best
     ;; handled by a out-of-line call on account of the extra hair.
     ;;
-    (defun cmacro-check-ds-list/&key (input min max pattern valid-keys)
-      (declare (type index min max) (optimize speed))
-      (flet ((require-constant-keys (plist)
-               ;; Signal a condition if the compiler should give up
-               ;; on expanding this macro. Well-formedness of the plist
-               ;; makes no difference, since CHECK-PLIST is rigorous.
-               ;; If the condition isn't handled, we just press onward.
-               (loop
-                  (when (atom plist) (return))
-                  (let ((key (pop plist)))
-                    (unless (or (keywordp key)
-                                (and (symbolp key)
-                                     (constantp key)
-                                     (eq key (symbol-value key))))
-                      (signal 'compiler-macro-keyword-problem :argument key)))
-                  (when (atom plist) (return))
-                  (pop plist))))
-        (scan-req-opt (input min max pattern list nil)
-                      :if-list-exhausted input
-                      :if-max-reached (progn (require-constant-keys list)
-                                             (check-plist input list valid-keys
-                                                          pattern)))))))
+  (defun cmacro-check-ds-list/&key (input min max pattern valid-keys)
+    (declare (type index min max) (optimize speed))
+    (scan-req-opt (input min max pattern list nil)
+                  :if-list-exhausted input
+                  :if-max-reached
+                  ;; Signal a condition if the compiler should give up
+                  ;; on expanding. Well-formedness of the plist
+                  ;; makes no difference, since CHECK-DS-BIND-KEYS is stricter.
+                  ;; If the condition isn't handled, we just press onward.
+                  (let ((plist list))
+                    (loop (when (atom plist) (return))
+                          (let ((key (pop plist)))
+                            (when (atom plist) (return))
+                            (pop plist)
+                            (unless (or (keywordp key)
+                                        (and (symbolp key)
+                                             (constantp key)
+                                             (eq key (symbol-value key))))
+                              (signal 'compiler-macro-keyword-problem
+                                      :argument key))))
+                    (check-ds-bind-keys input list valid-keys pattern)))))
 
 ;; Like GETF but return CDR of the cell whose CAR contained the found key,
 ;; instead of CADR; and return 0 for not found.
@@ -920,4 +1017,126 @@
            ;; Typecheck the next cell so that calling code doesn't get an atom.
            (return (the cons (cdr plist)))))))
 
+;;; This is a variant of destructuring-bind that provides the name
+;;; of the containing construct in generated error messages.
+(def!macro named-ds-bind (context lambda-list data &body body &environment env)
+  (declare (ignorable env))
+  `(binding* ,(expand-ds-bind lambda-list data t nil context
+                              (and (eq (car context) :macro)
+                                   (eq (cddr context) 'deftype)
+                                   ''*))
+     ,@body))
+
+;;; Make a lambda expression that receives an s-expression, destructures it
+;;; according to LAMBDA-LIST, and executes BODY.
+;;; NAME and KIND provide error-reporting context.
+;;; DOC-STRING-ALLOWED can be :INTERNAL to allow a docstring which is kept
+;;; inside the lambda, or :EXTERNAL to pull it out and return it, or NIL.
+;;; ENVIRONMENT can be NIL to disallow an &ENVIRONMENT variable,
+;;; or :IGNORE to allow it, but bind the corresponding symbol to NIL.
+;;; WRAP-BLOCK, if true, will place a block named NAME around body.
+;;;
+;;; The secondary value is a docstring, if requested as :EXTERNAL.
+;;;
+;;; The lambda contains an internal declaration of its argument list
+;;; that discards &ENVIRONMENT, &WHOLE, and/or anything else that does
+;;; not document the expected list shape.
+;;;
+;;; The CLtl2 name for this operation is PARSE-MACRO.
+(defun make-macro-lambda
+    (lambda-name lambda-list body kind name
+     &key ((:environment envp) t) (doc-string-allowed :internal)
+           (wrap-block name))
+  (declare (type (member t nil :ignore) envp))
+  (declare (type (member nil :external :internal) doc-string-allowed))
+  (binding* (((forms decls docstring)
+              (parse-body body :doc-string-allowed doc-string-allowed))
+             ;; Parse the lambda list, but not recursively.
+             ((llks req opt rest keys aux env whole)
+              (parse-lambda-list
+               lambda-list
+               :accept (logior
+                        (if envp (lambda-list-keyword-mask '&environment) 0)
+                        (lambda-list-keyword-mask 'destructuring-bind))
+               ;; Why :silent? We first parse to deconstruct and reconstruct
+               ;; without &WHOLE and &ENV, which is an implementation detail.
+               ;; When it comes to actually processing the entire lambda
+               ;; list again, that's when any warning(s) will be issued.
+               :context :macro :silent t))
+             ((outer-decls decls) (extract-var-decls decls (append env whole)))
+             (ll-env (when (eq envp t) (or env (list (make-symbol "ENV")))))
+             ;; We want a hidden WHOLE arg for the lambda - not the user's -
+             ;; in case one was present and declared IGNORE.
+             ;; Conversely, if the user asks for &WHOLE, doesn't use it,
+             ;; and doesn't declare it ignored, that deserves a warning.
+             (ll-whole (make-symbol "EXPR"))
+             ;; Then bind the user's WHOLE from the lambda's.
+             (ll-aux
+              (append (when (and (eq envp :ignore) env) `((,(car env) nil)))
+                      (when whole `((,(car whole) ,ll-whole)))))
+             ;; Drop &WHOLE and &ENVIRONMENT
+             (new-ll (make-lambda-list llks nil req opt rest keys aux))
+             (parse (parse-ds-lambda-list new-ll))
+             (arg-accessor
+              (if (eq kind 'define-compiler-macro) 'compiler-macro-args 'cdr)))
+    ;; Signal a style warning for duplicate names, but disregard &AUX variables
+    ;; because most folks agree that (LET* ((X (F)) (X (G X))) ..) makes sense
+    ;; - some would even say that it is idiomatic - and &AUX bindings are just
+    ;; LET* bindings.
+    ;; The obsolete PARSE-DEFMACRO signaled an error, but that seems harsh.
+    ;; Other implementations permit (X &OPTIONAL X), and the fact that
+    ;; nesting is allowed makes this issue even less clear.
+    (mapl (lambda (tail)
+            (when (memq (car tail) (cdr tail))
+              (style-warn "variable ~S occurs more than once" (car tail))))
+          (append whole env (ds-lambda-list-variables parse nil)))
+    (values `(,@(if lambda-name `(named-lambda ,lambda-name) '(lambda))
+                  (,ll-whole ,@ll-env ,@(and ll-aux (cons '&aux ll-aux)))
+               ,@(when (and docstring (eq doc-string-allowed :internal))
+                   (prog1 (list docstring) (setq docstring nil)))
+               ;; Normalize the lambda list by unparsing.
+               (declare (lambda-list ,(unparse-ds-lambda-list parse)))
+               ,@(if outer-decls (list outer-decls))
+               ,@(and (not env) (eq envp t) `((declare (ignore ,@ll-env))))
+               ,@(sb!c:macro-policy-decls)
+               (,@(if kind
+                      `(named-ds-bind ,(if (eq kind :special-form)
+                                           `(:special-form . ,name)
+                                           `(:macro ,name . ,kind)))
+                      '(destructuring-bind))
+                   ,new-ll (,arg-accessor ,ll-whole)
+                 ,@decls
+                 ,@(if wrap-block
+                       `((block ,(fun-name-block-name name) ,@forms))
+                       forms)))
+            docstring)))
+
+;;; Functions should probably not retain &AUX variables as part
+;;; of their reflected lambda list, but this is selectable
+;;; because some users might claim that dropping &AUX is wrong.
+;;; For system code, it's a measurably large waste of space,
+;;; given how DEFTRANSFORM and a few other macros expand such
+;;; that argument parsing is performed in &AUX var initforms.
+(defvar *strip-lamba-list-retain-aux* #+sb-xc t #-sb-xc nil)
+
+;;; Return LAMBDA-LIST with some pieces removed.
+(defun strip-lambda-list (lambda-list how)
+  (handler-case (parse-lambda-list lambda-list :silent t)
+   (error () lambda-list)
+   (:no-error (llks req opt rest keys aux &rest ignore)
+     (declare (ignore ignore))
+     (multiple-value-bind (opt keys aux)
+         (ecase how
+          (:arglist
+           (values opt keys (if *strip-lamba-list-retain-aux* aux nil)))
+          ;; The name of an anonymous lambda is an arbitrary list,
+          ;; not necessarily the original list.
+          (:name (values (mapcar #'parse-optional-arg-spec opt); Keep name.
+                         (mapcar #'parse-key-arg-spec keys) ; Keep keyword.
+                         nil))) ; Discard &AUX vars
+       (let ((new (make-lambda-list llks nil req opt rest keys aux)))
+         ;; It is harmful to the space-saving effect of this function
+         ;; if reconstituting the list results in an unnecessary copy.
+         (if (equal new lambda-list) lambda-list new))))))
+
 (/show0 "parse-lambda-list.lisp end of file")
