@@ -128,7 +128,7 @@
          (let ((class (find-class type nil)))
            (if class
                (let ((type (specializer-type class)))
-                 (if (listp type) type `(,type)))
+                 (ensure-list type))
                `(,type))))
         ((or (not (eq **boot-state** 'complete))
              (specializerp type))
@@ -207,7 +207,7 @@
 
 ;;;; built-in classes
 
-;;; Grovel over SB-KERNEL::*BUILT-IN-CLASSES* in order to set
+;;; Grovel over SB-KERNEL::*!BUILT-IN-CLASSES* in order to set
 ;;; SB-PCL:*BUILT-IN-CLASSES*.
 (/show "about to set up SB-PCL::*BUILT-IN-CLASSES*")
 (defvar *built-in-classes*
@@ -262,7 +262,7 @@
                                  ;; BUILT-IN-CLASS list
                                  '(t function stream sequence
                                      file-stream string-stream)))
-                       sb-kernel::*built-in-classes*))))
+                       sb-kernel::*!built-in-classes*))))
 (/noshow "done setting up SB-PCL::*BUILT-IN-CLASSES*")
 
 ;;;; the classes that define the kernel of the metabraid
@@ -459,7 +459,8 @@
     ;; KLUDGE: we need a reader for bootstrapping purposes, in
     ;; COMPUTE-EFFECTIVE-SLOT-DEFINITION-INITARGS.
     :reader %slot-definition-documentation)
-   (%class :initform nil :initarg :class :accessor slot-definition-class)))
+   (%class :initform nil :initarg :class :accessor slot-definition-class)
+   (source :initform nil :initarg source :accessor definition-source)))
 
 (defclass standard-slot-definition (slot-definition)
   ((allocation
