@@ -153,7 +153,7 @@
   (let* ((accept-inner
           (sb-int:lambda-list-keyword-mask 'destructuring-bind))
          (accept-outer
-          (logior (sb-int:lambda-list-keyword-mask '(&environment))
+          (logior (sb-int:lambda-list-keyword-mask '&environment)
                   accept-inner)))
     (sb-c::parse-lambda-list '(&whole w a b x) :accept accept-outer)
     (sb-c::parse-lambda-list '(&whole (w) a b x) :accept accept-inner)
@@ -185,12 +185,11 @@
          '((&body foo) (&whole (a . d) x y)))
 
     (try '(&optional a ((bb1 bb2) (f)) (c 'c) (d 'd dsp) &aux foo (baz))
-         '(&optional a ((bb1 bb2)) (c) (d)))
+         '(&optional a ((bb1 bb2)) (c 'c) (d 'd)))
 
     (try '(&key ((:bork (zook mook)) def bsp) (e 'e esp)
                 ((:name fred)) (color x csp))
-         '(&key ((:bork (zook mook))) (e) ((:name fred)) (color)))
-
+         '(&key ((:bork (zook mook))) (e 'e) ((:name fred)) (color)))
 
     (try '(x &optional (y) (((&whole (&whole w z . r) &body b) (c)) (def)))
          ;;                           ^ this &WHOLE variable is irrelevant
@@ -237,12 +236,12 @@
                    (cdr (parse '(a b c &environment foo . rest)))))
     (assert-error (parse '(a b c &environment foo &rest r . rest)))
 
-    ;; lp# 707556 will be fixed once PARSE-DEFMACRO uses PARSE-LAMBDA-LIST
+    ;; lp# 707556
     (assert-error (parse '(a &key b &allow-other-keys c)))))
 
 (with-test (:name :ds-lambda-list-symbols)
   (flet ((try (list expect)
-           (assert (equal sb-c::(ds-lambda-list-symbols
+           (assert (equal sb-c::(ds-lambda-list-variables
                                  (parse-ds-lambda-list list))
                          expect))))
     (try '(a ((b c))
@@ -255,6 +254,14 @@
 
 (with-test (:name :ds-lambda-list-possible-mismatch-warning)
   (assert-signal (sb-c::parse-ds-lambda-list '(a &optional ((b c) 'foo)))
+                 style-warning)
+  ;; Suppose this meant: one required arg followed by one optional arg
+  ;; which is a list of one required and one optional arg.
+  ;; But it's accidentally missing a pair of disambiguating parentheses.
+  (assert-signal (sb-c::parse-ds-lambda-list '(a &optional (b &optional c)))
+                 style-warning)
+  ;; Similarly...
+  (assert-signal (sb-c::parse-ds-lambda-list '(a &optional (b c &key)))
                  style-warning))
 
 (with-test (:name :ds-bind-list-checkers)
@@ -289,7 +296,7 @@
                        (lambda (args)
                          (sb-int:binding*
                              ,(sb-c::expand-ds-bind lambda-list 'args nil 'the)
-                           (list ,@(sb-c::ds-lambda-list-symbols
+                           (list ,@(sb-c::ds-lambda-list-variables
                                     (sb-c::parse-ds-lambda-list lambda-list))))))
                       (ast (sb-c::meta-abstractify-ds-lambda-list
                             (sb-c::parse-ds-lambda-list ',lambda-list))))
@@ -432,7 +439,13 @@
                      (lambda (c)
                        (declare (ignore c))
                        (return-from found
-                         (assoc 'sb-kernel::arg-count-error
+                         (assoc 'sb-c::ds-bind-error
                                 (sb-debug::list-backtrace))))))
-       (sb-kernel::arg-count-error 'destructuring-bind 'nil
-                                   '(1 2) '(a b c) 3 3))))))
+       (sb-c::ds-bind-error '(foo) 2 3 '((:macro baz . deftype))))))))
+
+(with-test (:name :destructuring-optional/key-warn-once-only)
+  (let ((count 0))
+    (handler-bind ((warning (lambda (c) (incf count) (muffle-warning c))))
+      (macroexpand-1
+       '(defmacro defx (name ll &optional (types '*) &key node) 1)))
+    (assert (= count 1))))
