@@ -706,4 +706,10 @@
            (funcall (compile nil `(lambda (x) (format nil "~s" (the string x))))
                     "\\")
            (prin1-to-string "\\"))))
-;;; success
+
+(with-test (:name :write-stream-nil)
+  (assert
+   (equal
+    (with-output-to-string (*standard-output*)
+      (funcall (compile nil `(lambda () (write "xx" :stream nil)))))
+    "\"xx\"")))
